#!/usr/bin/python
# -*- coding: utf-8 -*-

from thumbor.loaders import LoaderResult
from thumbor.utils import logger

import thumbor_gcs.client


async def load(context, path):
    """
    path is real storage file path
    for example:
        public/article/content/202205/07902518-0be0-4a60-8789-66d8b31665db.jpeg
    """
    logger.debug("[RESULT_STORAGE] loader origin path is %s" % path)
    path = path.lstrip('/')

    result = LoaderResult()

    if thumbor_gcs.client.Instance is None:
        thumbor_gcs.client.Instance = thumbor_gcs.client.Client(context)

    blob = thumbor_gcs.client.Instance.loader_get_object(path)
    if blob is None:
        """ file do not exist """
        result.error = LoaderResult.ERROR_NOT_FOUND
        result.successful = False
    else:
        """ file exist """
        result.successful = True
        result.buffer = blob.download_as_bytes()

        result.metadata.update(
            size=blob.size,
            updated_at=blob.updated,
        )

    return result



__version__ = "0.0.3"

__title__ = "thumbor-gcs"
__description__ = "Thumbor Loader and Result Storage for Google Cloud Storage(GCS)"
__uri__ = "https://github.com/jjonline/thumbor-gcs"
__doc__ = __description__ + " <" + __uri__ + ">"

__author_name__ = "Jea Yang"
__author_email__ = "jjonline@jjonline.cn"
__maintainer_name__ = "Jea Yang"
__maintainer_email__ = "jjonline@jjonline.cn"

__license__ = "Apache-2.0 license"
__copyright__ = "Copyright (c) 2022 Jea Yang"



#!/usr/bin/python
# -*- coding: utf-8 -*-

import hashlib
from urllib.parse import unquote

from google.cloud import storage
from google.cloud.storage import Client, Bucket
from thumbor.context import Context


class BucketClient:
    loaderClient: Client = None
    loaderBucket: Bucket = None
    resultClient: Client = None
    resultBucket: Bucket = None
    context: Context = None

    def __init__(self, context):
        """ init GCS when thumbor load thumbor_gcs.loader.gcs_loader"""
        self.context = context
        loader_project_id = context.config.get("LOADER_GCS_PROJECT_ID")
        loader_bucket_id = context.config.get("LOADER_GCS_BUCKET_ID")
        self.loaderClient = storage.Client(loader_project_id)
        self.loaderBucket = self.loaderClient.bucket(loader_bucket_id)

        result_project_id = self.context.config.get("RESULT_STORAGE_GCS_PROJECT_ID")
        result_bucket_id = self.context.config.get("RESULT_STORAGE_GCS_BUCKET_ID")
        """ if loader bucket and result_storage bucket are same, use same instance """
        if result_project_id == loader_project_id and result_bucket_id == loader_bucket_id:
            self.resultClient = self.loaderClient
            self.resultBucket = self.loaderBucket
        else:
            self.resultClient = storage.Client(result_project_id)
            self.resultBucket = self.loaderClient.bucket(result_bucket_id)

    def loader_get_object(self, path: str):
        """Get object from loader bucket.

        :type path: str
        :param path:
            REAL object path in GCS bucket.
            REAL object path is deal from the request PATH of URL.
            for example:
                `https://domain.com/public/sample.png`
                PATH of URL is `/public/sample.png`
                if your config `LOADER_GCS_ROOT_PATH` is EMPTY, then REAL object path is `public/sample.png`
                if your config `LOADER_GCS_ROOT_PATH` is SOME, then REAL object path is `SOME/public/sample.png`
        """
        root_path = self.context.config.LOADER_GCS_ROOT_PATH.strip("/")
        path = f"{root_path}/{path}".lstrip('/')
        return self.loaderBucket.get_blob(path)

    def result_get_object(self, path: str):
        """get object from result bucket.

        :type path: str
        :param path:
            The request path is generated by a certain algorithm, see method normalize_path
        """
        return self.resultBucket.get_blob(path)

    def result_put_object(self, path: str, stream: str or bytes, mime_type: str):
        """put object to result bucket.

        :type path: str
        :param path:
            The request path is generated by a certain algorithm, see method normalize_path

        :type stream: str or bytes
        :param stream:
            file stream str or bytes generate by thumbor core

        :type mime_type: str
        :param mime_type:
            file content-type, determine by thumbor core
        """
        blob = self.resultBucket.blob(path)
        blob.upload_from_string(stream)
        blob.cache_control = "public,max-age=%s" % self.context.config.MAX_AGE
        blob.content_type = mime_type
        blob.patch()
        return path


""" hold global BucketClient instance """
Instance: BucketClient or None = None



#!/usr/bin/python
# -*- coding: utf-8 -*-
import hashlib
from urllib.parse import unquote

from thumbor.result_storages import BaseStorage, ResultStorageResult
from thumbor.utils import logger, deprecated
from thumbor.engines import BaseEngine

import thumbor_gcs.client


class Storage(BaseStorage):

    def __init__(self, context):
        BaseStorage.__init__(self, context)
        """ set instance to global """
        thumbor_gcs.client.Instance = thumbor_gcs.client.BucketClient(context)

    @property
    def is_auto_webp(self):
        return self.context.config.AUTO_WEBP and self.context.request.accepts_webp

    def normalize_path(self, request_path):
        """put object to result bucket.

        :type request_path: str
        :param request_path:
            full PATH of request URL,
            for example:
                URL is `https://domain.com/security/0x0/public/sample.png`
                Then PATH is `/security/0x0/public/sample.png`
        """
        digest = hashlib.sha1(unquote(request_path).encode("utf-8")).hexdigest()

        """
        To avoid confusion when using the same GCS, you can set RESULT_STORAGE_GCS_ROOT_PATH value
        """
        root_path = self.context.config.RESULT_STORAGE_GCS_ROOT_PATH.rstrip("/")
        prefix = "auto_webp" if self.is_auto_webp else "default"

        return f"{root_path}/{prefix}/{digest[:2]}/{digest[2:4]}/{digest[4:]}"

    async def put(self, stream):
        path = self.normalize_path(self.context.request.url)
        try:
            logger.debug("[RESULT_STORAGE] put request URL path is %s" % self.context.request.url)
            logger.debug("[RESULT_STORAGE] put result FILE dir is %s" % path)
            return thumbor_gcs.client.Instance.result_put_object(path, stream, BaseEngine.get_mimetype(stream))
        except Exception as e:
            logger.error(f"[RESULT_STORAGE] put fatal {str(e)} at path {path}")
            return None

    async def get(self):
        """
        value of path example
        /-MvDe9B1kd5wNuDVcuXOqB2PRUs=/0x0/public/article/07902518-0be0-4a60-8789-66d8b31665db.jpeg
        """
        path = self.normalize_path(self.context.request.url)
        try:
            logger.debug("[RESULT_STORAGE] get request URL path is %s" % self.context.request.url)
            logger.debug("[RESULT_STORAGE] get result FILE dir is %s" % path)

            blob = thumbor_gcs.client.Instance.result_get_object(path)
            if blob is None:
                return None

            buffer = blob.download_as_bytes()
            result = ResultStorageResult(
                buffer=buffer,
                metadata={
                    "LastModified": blob.updated,
                    "ContentLength": blob.size,
                    "ContentType": BaseEngine.get_mimetype(buffer),
                },
            )

            return result
        except Exception as e:
            logger.debug(f"[RESULT_STORAGE] get result error {str(e)} at path {path}")
            return None

    @deprecated("Use result's last_modified instead")
    async def last_updated(self):
        path = self.normalize_path(self.context.request.url)
        blob = thumbor_gcs.client.Instance.result_get_object(path)
        if blob is None:
            logger.debug("[RESULT_STORAGE] method last_updated storage not found at %s" % path)
            return True

        return blob.updated
